@@ -8,11 +8,19 @@ data object.
 
 module Hledger.Data.Journal
 where
-import qualified Data.Map as Map
+import Data.List
 import Data.Map (findWithDefault, (!))
+import Data.Ord
+import Data.Time.Calendar
+import Data.Time.LocalTime
+import Data.Tree
 import Safe (headDef)
 import System.Time (ClockTime(TOD))
-import Hledger.Data.Utils
+import Test.HUnit
+import Text.Printf
+import qualified Data.Map as Map
+
+import Hledger.Utils
 import Hledger.Data.Types
 import Hledger.Data.AccountName
 import Hledger.Data.Amount
@@ -21,6 +29,7 @@ import Hledger.Data.Dates (nulldatespan)
 import Hledger.Data.Transaction (journalTransactionWithDate,balanceTransaction)
 import Hledger.Data.Posting
 import Hledger.Data.TimeLog
+import Hledger.Data.Matching
 
 
 instance Show Journal where
@@ -34,6 +43,18 @@ instance Show Journal where
              -- ++ (show $ journalTransactions l)
              where accounts = flatten $ journalAccountNameTree j
 
+showJournalDebug j = unlines [
+                      show j
+                     ,show (jtxns j)
+                     ,show (jmodifiertxns j)
+                     ,show (jperiodictxns j)
+                     ,show $ open_timelog_entries j
+                     ,show $ historical_prices j
+                     ,show $ final_comment_lines j
+                     ,show $ jContext j
+                     ,show $ map fst $ files j
+                     ]
+
 nulljournal :: Journal
 nulljournal = Journal { jmodifiertxns = []
                       , jperiodictxns = []
@@ -47,17 +68,16 @@ nulljournal = Journal { jmodifiertxns = []
                       }
 
 nullctx :: JournalContext
-nullctx = Ctx { ctxYear = Nothing, ctxCommodity = Nothing, ctxAccount = [] }
+nullctx = Ctx { ctxYear = Nothing, ctxCommodity = Nothing, ctxAccount = [], ctxAliases = [] }
 
+nullfilterspec :: FilterSpec
 nullfilterspec = FilterSpec {
      datespan=nulldatespan
     ,cleared=Nothing
     ,real=False
     ,empty=False
-    ,costbasis=False
     ,acctpats=[]
     ,descpats=[]
-    ,whichdate=ActualDate
     ,depth=Nothing
     }
 
@@ -89,7 +109,7 @@ journalPostings :: Journal -> [Posting]
 journalPostings = concatMap tpostings . jtxns
 
 journalAccountNamesUsed :: Journal -> [AccountName]
-journalAccountNamesUsed = accountNamesFromPostings . journalPostings
+journalAccountNamesUsed = sort . accountNamesFromPostings . journalPostings
 
 journalAccountNames :: Journal -> [AccountName]
 journalAccountNames = sort . expandAccountNames . journalAccountNamesUsed
@@ -100,38 +120,49 @@ journalAccountNameTree = accountNameTreeFrom . journalAccountNames
 -- Various kinds of filtering on journals. We do it differently depending
 -- on the command.
 
--- | Keep only transactions we are interested in, as described by
--- the filter specification. May also massage the data a little.
+-------------------------------------------------------------------------------
+-- filtering V2
+
+-- | Keep only postings matching the query expression.
+-- This can leave unbalanced transactions.
+filterJournalPostings2 :: Matcher -> Journal -> Journal
+filterJournalPostings2 m j@Journal{jtxns=ts} = j{jtxns=map filtertransactionpostings ts}
+    where
+      filtertransactionpostings t@Transaction{tpostings=ps} = t{tpostings=filter (m `matchesPosting`) ps}
+
+-- | Keep only transactions matching the query expression.
+filterJournalTransactions2 :: Matcher -> Journal -> Journal
+filterJournalTransactions2 m j@Journal{jtxns=ts} = j{jtxns=filter (m `matchesTransaction`) ts}
+
+-------------------------------------------------------------------------------
+-- filtering V1
+
+-- | Keep only transactions we are interested in, as described by the
+-- filter specification.
 filterJournalTransactions :: FilterSpec -> Journal -> Journal
 filterJournalTransactions FilterSpec{datespan=datespan
                                     ,cleared=cleared
                                     -- ,real=real
                                     -- ,empty=empty
-                                    -- ,costbasis=_
                                     ,acctpats=apats
                                     ,descpats=dpats
-                                    ,whichdate=whichdate
                                     ,depth=depth
                                     } =
     filterJournalTransactionsByClearedStatus cleared .
     filterJournalPostingsByDepth depth .
     filterJournalTransactionsByAccount apats .
     filterJournalTransactionsByDescription dpats .
-    filterJournalTransactionsByDate datespan .
-    journalSelectingDate whichdate
+    filterJournalTransactionsByDate datespan
 
--- | Keep only postings we are interested in, as described by
--- the filter specification. May also massage the data a little.
--- This can leave unbalanced transactions.
+-- | Keep only postings we are interested in, as described by the filter
+-- specification. This can leave unbalanced transactions.
 filterJournalPostings :: FilterSpec -> Journal -> Journal
 filterJournalPostings FilterSpec{datespan=datespan
                                 ,cleared=cleared
                                 ,real=real
                                 ,empty=empty
---                                ,costbasis=costbasis
                                 ,acctpats=apats
                                 ,descpats=dpats
-                                ,whichdate=whichdate
                                 ,depth=depth
                                 } =
     filterJournalPostingsByRealness real .
@@ -140,8 +171,7 @@ filterJournalPostings FilterSpec{datespan=datespan
     filterJournalPostingsByDepth depth .
     filterJournalPostingsByAccount apats .
     filterJournalTransactionsByDescription dpats .
-    filterJournalTransactionsByDate datespan .
-    journalSelectingDate whichdate
+    filterJournalTransactionsByDate datespan
 
 -- | Keep only transactions whose description matches the description patterns.
 filterJournalTransactionsByDescription :: [String] -> Journal -> Journal
@@ -208,7 +238,7 @@ filterJournalTransactionsByAccount apats j@Journal{jtxns=ts} = j{jtxns=filter tm
       tmatch t = (null positives || any positivepmatch ps) && (null negatives || not (any negativepmatch ps)) where ps = tpostings t
       positivepmatch p = any (`amatch` a) positives where a = paccount p
       negativepmatch p = any (`amatch` a) negatives where a = paccount p
-      amatch pat a = containsRegex (abspat pat) a
+      amatch pat a = regexMatchesCI (abspat pat) a
       (negatives,positives) = partition isnegativepat apats
 
 -- | Keep only postings which affect accounts matched by the account patterns.
@@ -224,6 +254,13 @@ journalSelectingDate ActualDate j = j
 journalSelectingDate EffectiveDate j =
     j{jtxns=map (journalTransactionWithDate EffectiveDate) $ jtxns j}
 
+-- | Apply additional account aliases (eg from the command-line) to all postings in a journal.
+journalApplyAliases :: [(AccountName,AccountName)] -> Journal -> Journal
+journalApplyAliases aliases j@Journal{jtxns=ts} = j{jtxns=map fixtransaction ts}
+    where
+      fixtransaction t@Transaction{tpostings=ps} = t{tpostings=map fixposting ps}
+      fixposting p@Posting{paccount=a} = p{paccount=accountNameApplyAliases aliases a}
+
 -- | Do post-parse processing on a journal, to make it ready for use.
 journalFinalise :: ClockTime -> LocalTime -> FilePath -> String -> JournalContext -> Journal -> Either String Journal
 journalFinalise tclock tlocal path txt ctx j@Journal{files=fs} =
@@ -291,7 +328,7 @@ journalConvertAmountsToCost j@Journal{jtxns=ts} = j{jtxns=map fixtransaction ts}
       fixtransaction t@Transaction{tpostings=ps} = t{tpostings=map fixposting ps}
       fixposting p@Posting{pamount=a} = p{pamount=fixmixedamount a}
       fixmixedamount (Mixed as) = Mixed $ map fixamount as
-      fixamount = canonicaliseAmount (Just $ journalCanonicalCommodities j) . costOfAmount
+      fixamount = canonicaliseAmountCommodity (Just $ journalCanonicalCommodities j) . costOfAmount
 
 -- | Get this journal's unique, display-preference-canonicalised commodities, by symbol.
 journalCanonicalCommodities :: Journal -> Map.Map String Commodity
@@ -334,7 +371,7 @@ matchpats pats str =
     where
       (negatives,positives) = partition isnegativepat pats
       match "" = True
-      match pat = containsRegex (abspat pat) str
+      match pat = regexMatchesCI (abspat pat) str
 
 negateprefix = "not:"
 
@@ -390,6 +427,10 @@ postingsByAccount ps = m'
       groupedps = groupBy (\p1 p2 -> paccount p1 == paccount p2) sortedps
       m' = Map.fromList [(paccount $ head g, g) | g <- groupedps]
 
+-- debug helpers
+traceAmountPrecision a = trace (show $ map (precision . commodity) $ amounts a) a
+tracePostingsCommodities ps = trace (show $ map ((map (precision . commodity) . amounts) . pamount) ps) ps
+
 tests_Hledger_Data_Journal = TestList [
  ]
 
