@@ -10,6 +10,7 @@ module Hledger.Data.Transaction
 where
 import Data.List
 import Data.Maybe
+import Data.Time.Calendar
 import Test.HUnit
 import Text.Printf
 import qualified Data.Map as Map
@@ -59,27 +60,23 @@ pcommentwidth = no limit -- 22
 @
 -}
 showTransaction :: Transaction -> String
-showTransaction = showTransaction' True False
+showTransaction = showTransaction' True
 
 showTransactionUnelided :: Transaction -> String
-showTransactionUnelided = showTransaction' False False
+showTransactionUnelided = showTransaction' False
 
-showTransactionForPrint :: Bool -> Transaction -> String
-showTransactionForPrint effective = showTransaction' False effective
-
-showTransaction' :: Bool -> Bool -> Transaction -> String
-showTransaction' elide effective t =
+showTransaction' :: Bool -> Transaction -> String
+showTransaction' elide t =
     unlines $ [description] ++ showpostings (tpostings t) ++ [""]
     where
       description = concat [date, status, code, desc, comment]
-      date | effective = showdate $ fromMaybe (tdate t) $ teffectivedate t
-           | otherwise = showdate (tdate t) ++ maybe "" showedate (teffectivedate t)
+      date = showdate (tdate t) ++ maybe "" showedate (teffectivedate t)
+      showdate = printf "%-10s" . showDate
+      showedate = printf "=%s" . showdate
       status = if tstatus t then " *" else ""
       code = if length (tcode t) > 0 then printf " (%s)" $ tcode t else ""
       desc = if null d then "" else " " ++ d where d = tdescription t
       comment = if null c then "" else "  ; " ++ c where c = tcomment t
-      showdate = printf "%-10s" . showDate
-      showedate = printf "=%s" . showdate
       showpostings ps
           | elide && length ps > 1 && isTransactionBalanced Nothing t -- imprecise balanced check
               = map showposting (init ps) ++ [showpostingnoamt (last ps)]
@@ -237,10 +234,18 @@ nonzerobalanceerror t = printf "could not balance this transaction (%s%s%s)" rms
             | otherwise = "balanced virtual postings are off by " ++ show (costOfMixedAmount bvsum)
       sep = if not (null rmsg) && not (null bvmsg) then "; " else ""
 
--- | Convert the primary date to either the actual or effective date.
+transactionActualDate :: Transaction -> Day
+transactionActualDate = tdate
+
+-- Get a transaction's effective date, defaulting to the actual date.
+transactionEffectiveDate :: Transaction -> Day
+transactionEffectiveDate t = fromMaybe (tdate t) $ teffectivedate t
+
+-- | Once we no longer need both, set the main transaction date to either
+-- the actual or effective date. A bit hacky.
 journalTransactionWithDate :: WhichDate -> Transaction -> Transaction
 journalTransactionWithDate ActualDate t = t
-journalTransactionWithDate EffectiveDate t = txnTieKnot t{tdate=fromMaybe (tdate t) (teffectivedate t)}
+journalTransactionWithDate EffectiveDate t = txnTieKnot t{tdate=transactionEffectiveDate t}
 
 -- | Ensure a transaction's postings refer back to it.
 txnTieKnot :: Transaction -> Transaction
