@@ -4,15 +4,15 @@
 Most data types are defined here to avoid import cycles.
 Here is an overview of the hledger data model:
 
-> Journal                  -- a journal is derived from one or more data files. It contains..
->  [Transaction]           -- journal transactions, which have date, status, code, description and..
->   [Posting]              -- multiple account postings (entries), which have account name and amount.
+> Journal                  -- a journal is read from one or more data files. It contains..
+>  [Transaction]           -- journal transactions (aka entries), which have date, status, code, description and..
+>   [Posting]              -- multiple account postings, which have account name and amount
 >  [HistoricalPrice]       -- historical commodity prices
 >
 > Ledger                   -- a ledger is derived from a journal, by applying a filter specification and doing some further processing. It contains..
->  Journal                 -- the filtered journal, containing only the transactions and postings we are interested in
->  Tree AccountName        -- account names referenced in the journal's transactions, as a tree
->  Map AccountName Account -- per-account postings and balances from the journal's transactions, as a  map from account name to account info
+>  Journal                 -- a filtered copy of the original journal, containing only the transactions and postings we are interested in
+>  Tree AccountName        -- all accounts named by the journal's transactions, as a hierarchy
+>  Map AccountName Account -- the postings, and resulting balances, in each account
 
 For more detailed documentation on each type, see the corresponding modules.
 
@@ -33,12 +33,13 @@ Evolution of transaction\/entry\/posting terminology:
 module Hledger.Data.Types
 where
 import Control.Monad.Error (ErrorT)
-import Data.Typeable (Typeable)
+import Data.Time.Calendar
+import Data.Time.LocalTime
+import Data.Tree
+import Data.Typeable
 import qualified Data.Map as Map
 import System.Time (ClockTime)
 
-import Hledger.Data.Utils
-
 
 type SmartDate = (String,String,String)
 
@@ -68,9 +69,10 @@ data Commodity = Commodity {
       separatorpositions :: [Int]  -- ^ positions of separators, counting leftward from decimal point
     } deriving (Eq,Ord,Show,Read)
 
--- | An amount's price may be written as \@ unit price or \@\@ total price.
--- Note although Price has a MixedAmount, it should hold only
--- single-commodity amounts, cf costOfAmount.
+-- | An amount's price in another commodity may be written as \@ unit
+-- price or \@\@ total price.  Note although a MixedAmount is used, it
+-- should be in a single commodity, also the amount should be positive;
+-- these are not enforced currently.
 data Price = UnitPrice MixedAmount | TotalPrice MixedAmount
              deriving (Eq,Ord)
 
@@ -146,7 +148,10 @@ type Year = Integer
 data JournalContext = Ctx {
       ctxYear      :: !(Maybe Year)      -- ^ the default year most recently specified with Y
     , ctxCommodity :: !(Maybe Commodity) -- ^ the default commodity most recently specified with D
-    , ctxAccount   :: ![AccountName]     -- ^ the current stack of parent accounts specified by !account
+    , ctxAccount   :: ![AccountName]     -- ^ the current stack of parent accounts/account name components
+                                        --   specified with "account" directive(s). Concatenated, these
+                                        --   are the account prefix prepended to parsed account names.
+    , ctxAliases   :: ![(AccountName,AccountName)] -- ^ the current list of account name aliases in effect
     } deriving (Read, Show, Eq)
 
 data Journal = Journal {
@@ -187,16 +192,14 @@ data Account = Account {
       abalance :: MixedAmount    -- ^ sum of postings in this account and subaccounts
     }
 
--- | A generic, pure specification of how to filter transactions and postings.
+-- | A generic, pure specification of how to filter (or search) transactions and postings.
 data FilterSpec = FilterSpec {
      datespan  :: DateSpan   -- ^ only include if in this date span
     ,cleared   :: Maybe Bool -- ^ only include if cleared\/uncleared\/don't care
     ,real      :: Bool       -- ^ only include if real\/don't care
     ,empty     :: Bool       -- ^ include if empty (ie amount is zero)
-    ,costbasis :: Bool       -- ^ convert all amounts to cost basis
     ,acctpats  :: [String]   -- ^ only include if matching these account patterns
     ,descpats  :: [String]   -- ^ only include if matching these description patterns
-    ,whichdate :: WhichDate  -- ^ which dates to use (actual or effective)
     ,depth     :: Maybe Int
     } deriving (Show)
 
