@@ -146,10 +146,14 @@ clearedValueFromOpts ReportOpts{..} | cleared_   = Just True
                                     | uncleared_ = Just False
                                     | otherwise  = Nothing
 
--- | Detect which date we will report on, based on --effective.
+-- | Report which date we will report on based on --effective.
 whichDateFromOpts :: ReportOpts -> WhichDate
 whichDateFromOpts ReportOpts{..} = if effective_ then EffectiveDate else ActualDate
 
+-- | Select a Transaction date accessor based on --effective.
+transactionDateFn :: ReportOpts -> (Transaction -> Day)
+transactionDateFn ReportOpts{..} = if effective_ then transactionEffectiveDate else transactionActualDate
+
 -- | Convert this journal's transactions' primary date to either the
 -- actual or effective date, as per options.
 journalSelectingDateFromOpts :: ReportOpts -> Journal -> Journal
@@ -197,9 +201,10 @@ type EntriesReportItem = Transaction
 
 -- | Select transactions for an entries report.
 entriesReport :: ReportOpts -> FilterSpec -> Journal -> EntriesReport
-entriesReport opts fspec j = sortBy (comparing tdate) $ jtxns $ filterJournalTransactions fspec j'
+entriesReport opts fspec j = sortBy (comparing f) $ jtxns $ filterJournalTransactions fspec j'
     where
-      j' = journalSelectingDateFromOpts opts $ journalSelectingAmountFromOpts opts j
+      f = transactionDateFn opts
+      j' = journalSelectingAmountFromOpts opts j
 
 -------------------------------------------------------------------------------
 
@@ -219,17 +224,28 @@ postingsReport :: ReportOpts -> FilterSpec -> Journal -> PostingsReport
 postingsReport opts fspec j = (totallabel, postingsReportItems ps nullposting startbal (+))
     where
       ps | interval == NoInterval = displayableps
-         | otherwise              = summarisePostingsByInterval interval depth empty filterspan displayableps
-      (precedingps, displayableps, _) = postingsMatchingDisplayExpr (display_ opts)
+         | otherwise              = summarisePostingsByInterval interval depth empty reportspan displayableps
+      j' =                                journalSelectingDateFromOpts opts
+                                        $ journalSelectingAmountFromOpts opts
+                                        j
+      (precedingps, displayableps, _) =   postingsMatchingDisplayExpr (display_ opts)
                                         $ depthClipPostings depth
                                         $ journalPostings
                                         $ filterJournalPostings fspec{depth=Nothing}
-                                        $ journalSelectingDateFromOpts opts
-                                        $ journalSelectingAmountFromOpts opts
-                                        j
+                                        j'
+      (interval, depth, empty, displayexpr) = (intervalFromOpts opts, depth_ opts, empty_ opts, display_ opts)
+      journalspan = journalDateSpan j'
+      -- requestedspan should be the intersection of any span specified
+      -- with period options and any span specified with display option.
+      -- The latter is not easily available, fake it for now.
+      requestedspan = periodspan `spanIntersect` displayspan
+      periodspan = datespan fspec
+      displayspan = postingsDateSpan ps
+          where (_,ps,_) = postingsMatchingDisplayExpr displayexpr $ journalPostings j'
+      matchedspan = postingsDateSpan displayableps
+      reportspan | empty     = requestedspan `orDatesFrom` journalspan
+                 | otherwise = requestedspan `spanIntersect` matchedspan
       startbal = sumPostings precedingps
-      filterspan = datespan fspec
-      (interval, depth, empty) = (intervalFromOpts opts, depth_ opts, empty_ opts)
 
 totallabel = "Total"
 balancelabel = "Balance"
@@ -304,13 +320,10 @@ depthClipPosting (Just d) p@Posting{paccount=a} = p{paccount=clipAccountName d a
 -- are one per account per interval and aggregated to the specified depth
 -- if any.
 summarisePostingsByInterval :: Interval -> Maybe Int -> Bool -> DateSpan -> [Posting] -> [Posting]
-summarisePostingsByInterval interval depth empty filterspan ps = concatMap summarisespan $ splitSpan interval reportspan
+summarisePostingsByInterval interval depth empty reportspan ps = concatMap summarisespan $ splitSpan interval reportspan
     where
       summarisespan s = summarisePostingsInDateSpan s depth empty (postingsinspan s)
       postingsinspan s = filter (isPostingInDateSpan s) ps
-      dataspan = postingsDateSpan ps
-      reportspan | empty = filterspan `orDatesFrom` dataspan
-                 | otherwise = dataspan
 
 -- | Given a date span (representing a reporting interval) and a list of
 -- postings within it: aggregate the postings so there is only one per
@@ -433,7 +446,7 @@ accountTransactionsReportItems matcher thisacctmatcher bal signfn (t:ts) =
       (psthisacct,psotheracct) = case thisacctmatcher of Just m  -> partition (matchesPosting m) psmatched
                                                          Nothing -> ([],psmatched)
       numotheraccts = length $ nub $ map paccount psotheracct
-      amt = sum $ map pamount psotheracct
+      amt = negate $ sum $ map pamount psthisacct
       acct | isNothing thisacctmatcher = summarisePostings psmatched -- journal register
            | numotheraccts == 0 = "transfer between " ++ summarisePostingAccounts psthisacct
            | otherwise          = prefix              ++ summarisePostingAccounts psotheracct
@@ -526,6 +539,7 @@ isInteresting :: ReportOpts -> Ledger -> AccountName -> Bool
 isInteresting opts l a | flat_ opts = isInterestingFlat opts l a
                        | otherwise = isInterestingIndented opts l a
 
+-- | Determine whether an account should get its own line in the --flat balance report.
 isInterestingFlat :: ReportOpts -> Ledger -> AccountName -> Bool
 isInterestingFlat opts l a = notempty || emptyflag
     where
@@ -533,16 +547,19 @@ isInterestingFlat opts l a = notempty || emptyflag
       notempty = not $ isZeroMixedAmount $ exclusiveBalance acct
       emptyflag = empty_ opts
 
+-- | Determine whether an account should get its own line in the indented
+-- balance report.  Cf Balance module doc.
 isInterestingIndented :: ReportOpts -> Ledger -> AccountName -> Bool
 isInterestingIndented opts l a
-    | numinterestingsubs==1 && not atmaxdepth = notlikesub
-    | otherwise = notzero || emptyflag
+    | numinterestingsubs == 1 && samebalanceassub && not atmaxdepth = False
+    | numinterestingsubs < 2 && zerobalance && not emptyflag = False
+    | otherwise = True
     where
       atmaxdepth = isJust d && Just (accountNameLevel a) == d where d = depth_ opts
       emptyflag = empty_ opts
       acct = ledgerAccount l a
-      notzero = not $ isZeroMixedAmount inclbalance where inclbalance = abalance acct
-      notlikesub = not $ isZeroMixedAmount exclbalance where exclbalance = sumPostings $ apostings acct
+      zerobalance = isZeroMixedAmount inclbalance where inclbalance = abalance acct
+      samebalanceassub = isZeroMixedAmount exclbalance where exclbalance = sumPostings $ apostings acct
       numinterestingsubs = length $ filter isInterestingTree subtrees
           where
             isInterestingTree = treeany (isInteresting opts l . aname)
